package com.alibaba.apm;

import io.opentracing.Scope;
import io.opentracing.Span;
import io.opentracing.Tracer;
import io.opentracing.util.GlobalTracer;

import static java.lang.System.exit;

public class OpenTracingDemo {

    static {
        io.jaegertracing.Configuration config = new io.jaegertracing.Configuration("manalDemo");
        io.jaegertracing.Configuration.SenderConfiguration sender = new io.jaegertracing.Configuration.SenderConfiguration();
        /**
         *  从https://tracing.console.aliyun.com/ -> 集群配置 -》接入点信息 获取jaeger的网关（Endpoint）
         *  运行前，请设置当前用户的对应的网关
         */
        sender.withEndpoint("http://tracing-analysis-dc-sh.aliyuncs.com/adapt_123@3456_222@444/api/traces");
        config.withSampler(new io.jaegertracing.Configuration.SamplerConfiguration().withType("const").withParam(1));
        config.withReporter(new io.jaegertracing.Configuration.ReporterConfiguration().withSender(sender).withMaxQueueSize(10000));
        GlobalTracer.registerIfAbsent(config.getTracer());
    }


    public static void main(String[] args) {
        System.out.println("begin tracing");
        OpenTracingDemo test = new OpenTracingDemo();
        test.firstBiz();

        try {
            Thread.sleep(10000);
        } catch (InterruptedException e) {
            // quite
        }
        exit(0);
    }

    private void firstBiz() {
        Tracer tracer = GlobalTracer.get();
        // 创建span
        Span span = tracer.buildSpan("parentSpan").withTag("myTag", "spanFrist").start();
        try (Scope ignored = tracer.activateSpan(span)) {

            tracer.activeSpan().setTag("methodName", "testTracing");
            // .... 业务逻辑
            secondBiz();
        } catch (Exception e) {
            TracingHelper.onError(e, span);
            throw e;
        } finally {
            span.finish();
        }
    }

    private void secondBiz() {
        Tracer tracer = GlobalTracer.get();
        Span childSpan = tracer.buildSpan("childSpan").withTag("myTag", "spanSecond").start();
        try (Scope ignored = tracer.activateSpan(childSpan)) {
            tracer.activeSpan().setTag("methodName", "testCall");
            //... 业务逻辑
            // mock exception
            //throw new RuntimeException("TestRuntimeException");
        } catch (Exception e) {
            TracingHelper.onError(e, childSpan);
            throw e;
        } finally {
            childSpan.finish();
        }


    }

}
