package org.webmvc;

import io.opentracing.Tracer;
import io.opentracing.contrib.apache.http.client.TracingHttpClientBuilder;
import io.opentracing.contrib.spring.web.client.TracingRestTemplateInterceptor;
import io.opentracing.contrib.spring.web.interceptor.TracingHandlerInterceptor;
import io.opentracing.util.GlobalTracer;
import org.apache.http.client.HttpClient;
import org.apache.http.client.config.RequestConfig;
import org.apache.http.impl.conn.PoolingHttpClientConnectionManager;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.context.annotation.Bean;
import org.springframework.context.annotation.Configuration;
import org.springframework.http.client.ClientHttpRequestInterceptor;
import org.springframework.web.client.RestTemplate;
import org.springframework.web.servlet.config.annotation.InterceptorRegistry;
import org.springframework.web.servlet.config.annotation.WebMvcConfigurerAdapter;

import java.util.ArrayList;
import java.util.List;


@Configuration
//@Import({TracingHandlerInterceptor.class})
public class AppConfiguration extends WebMvcConfigurerAdapter {
    private static Tracer SINGLE_TRACE = null;

    static {
        String serviceName = System.getProperty("openTracing.service");
        if (serviceName == null ||serviceName.length() ==0) {
            serviceName = "springmvc";
        }
        io.jaegertracing.Configuration config = new io.jaegertracing.Configuration(serviceName);
        io.jaegertracing.Configuration.SenderConfiguration sender = new io.jaegertracing.Configuration.SenderConfiguration();
        /**
         *  从https://tracing.console.aliyun.com/ -> 集群配置 -》接入点信息 获取jaeger的网关（Endpoint）
         *  第一次运行时，请设置当前用户的对应的网关
         */
        sender.withEndpoint("http://tracing-analysis-dc-sh.aliyuncs.com/adapt_123@3456_222@444/api/traces");
        config.withSampler(new io.jaegertracing.Configuration.SamplerConfiguration().withType("const").withParam(1));
        config.withReporter(new io.jaegertracing.Configuration.ReporterConfiguration().withSender(sender).withMaxQueueSize(10000));
        SINGLE_TRACE = config.getTracer();
        GlobalTracer.register(SINGLE_TRACE);
    }

    @Autowired
    private Tracer tracer;

    @Override
    public void addInterceptors(InterceptorRegistry registry) {
        registry.addInterceptor(new TracingHandlerInterceptor(tracer));
    }

    @Bean
    public io.opentracing.Tracer tracer() {
        return SINGLE_TRACE;
    }
    @Bean
    public HttpClient httpClient() {
        RequestConfig defaultRequestConfig = RequestConfig.custom()
                .setSocketTimeout(2000)
                .setConnectTimeout(3000)
                .setConnectionRequestTimeout(3000)
                .build();
        PoolingHttpClientConnectionManager connManager = new PoolingHttpClientConnectionManager();
        connManager.setMaxTotal(100);
        connManager.setDefaultMaxPerRoute(10);
        HttpClient httpClient =  TracingHttpClientBuilder.create().withTracer(tracer)
                .setDefaultRequestConfig(defaultRequestConfig)
                .setConnectionManager(connManager)
                .build();
        return httpClient;
    }

    public static Tracer getTrace() {
        return SINGLE_TRACE;
    }

    @Bean
    RestTemplate restTemplate() {
        RestTemplate restTemplate = new RestTemplate();
        List<ClientHttpRequestInterceptor> list = new ArrayList<>();
        list.add(new TracingRestTemplateInterceptor(tracer));
        restTemplate.setInterceptors(list);
        return restTemplate;
    }
}
